unit Scmain;
{
 Copier-Utility: CompuCopier
 This program demonstrate the use of the eztwain.dll (eztwain.pas).
 If you have a flatbed scanner and a graphics printer, you can use
 'Compu-Copier' to make copies of a sheet of paper in a easy way.

 known bugs:
 - colored copies are not possible, because of a bug in eztwain.dll
 - if the scanner doesn't deliver resolution information, the copy
   may not have the same size as the original (at 100% user scale)

 history:
 1.0a:  first release
 1.0b:  Divison by zero removed, if the TWAIN-Scanner doesn't deliver
        resolution information
 1.0c:  Better handling of errors, if the TWAIN-Scanner doesn't deliver
        resolution information
 1.0d:  try to close TWAIN, if it's already open
 1.0e:  DebugBox temporary added
 1.0f:  Errorhandling improved (div by zero)
 1.0g:  DebugBox removed
}
interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, EzTwain, Printers, ExtCtrls, Spin,
  Menus, Scoptio, Scabout, DebugBox;

type
  TForm1 = class(TForm)
    Button1: TButton;
    PrinterSetupDialog1: TPrinterSetupDialog;
    SpinEdit1: TSpinEdit;
    Label1: TLabel;
    MainMenu1: TMainMenu;
    MFile: TMenuItem;
    MPrintersettings: TMenuItem;
    MClose: TMenuItem;
    Label2: TLabel;
    SpinEdit2: TSpinEdit;
    MScannersettings: TMenuItem;
    Button2: TButton;
    Label3: TLabel;
    MOptions: TMenuItem;
    About1: TMenuItem;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure MPrintersettingsClick(Sender: TObject);
    procedure MCloseClick(Sender: TObject);
    procedure MScannersettingsClick(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure MOptionsClick(Sender: TObject);
    procedure About1Click(Sender: TObject);
  private
    { Private-Deklarationen }
    procedure UpDateInfo;
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

function DibNumColors(pv: pointer): word;
{given a pointer to a locked DIB, return the number of palette entries: 0,2,16, or 256}
var
    Bits: integer;
    lpbi: PBITMAPINFOHEADER;
    lpbc: PBITMAPCOREHEADER;
begin
    lpbi := PBITMAPINFOHEADER(pv);
    lpbc := PBITMAPCOREHEADER(pv);
    {
    /*    With the BITMAPINFO format headers, the size of the palette
     *    is in biClrUsed, whereas in the BITMAPCORE - style headers, it
     *    is dependent on the bits per pixel ( = 2 raised to the power of
     *    bits/pixel).
     */
    }
    if (lpbi^.biSize <> sizeof(TBITMAPCOREHEADER)) then
    begin
        if (lpbi^.biClrUsed <> 0) then
            Result := WORD(lpbi^.biClrUsed);
        Bits := lpbi^.biBitCount;
    end
    else
    begin
		Bits := lpbc^.bcBitCount;
	end;
    Result := (1 shl Bits) and $01ff; {up to 8 bits, 2 ^ Bits - otherwise, 0.}
end;

function LPBits(lpdib: PBITMAPINFOHEADER): pointer;
{ Given a pointer to a locked DIB, return a pointer to the actual bits (pixels) }
var
    dwColorTableSize: longint;
begin
    dwColorTableSize := longint( (DibNumColors(lpdib) * sizeof(TRGBQUAD)));
    lpBits := pointer( longint(lpdib) + lpdib^.biSize + dwColorTableSize);
end;

procedure PrintDIB(PrinterHandle: HDC; BHandle: HBitmap; UserScaleX, UserScaleY: Single; Center: TCenterState);
 function GetDibResX(Info: PBitmapInfoHeader): Single;
 begin {DIB-resolution in dpi}
    if (Info^.biXPelsPerMeter>0) and (Info^.biXPelsPerMeter<400000) then
       Result:=Info^.biXPelsPerMeter*25.4/1000 {Resolution in dpi}
    else
       Result:=0; {Resolution =0 or greater than 10000dpi}
 end;
 function GetDibResY(Info: PBitmapInfoHeader): Single;
 begin
    if (Info^.biYPelsPerMeter>0) and (Info^.biYPelsPerMeter<400000) then
       Result:=Info^.biYPelsPerMeter*25.4/1000 {Resolution in dpi}
    else
       Result:=0; {Resolution =0 or greater than 10000dpi}
 end;
 function GetPrnResX( h: HDC ): Single;
 begin {Printerresolution in dpi}
   if (GetDeviceCaps(h, logPixelsX)>0) and (GetDeviceCaps(h, logPixelsX)<10000) then
      Result:=GetDeviceCaps(h, logPixelsX)
   else
      Result:=0;
 end;
 function GetPrnResY( h: HDC ): Single;
 begin {Printerresolution in dpi}
   if (GetDeviceCaps(h, logPixelsY)>0) and (GetDeviceCaps(h, logPixelsY)<10000) then
      Result:=GetDeviceCaps(h, logPixelsY)
   else
      Result:=0;
 end;
 var
    Info: PBitmapInfoHeader;
    i: integer;
    x,y,w,h: longint;
    Offset, PageSize: TPoint;
    ScaleX, ScaleY: Single;
begin
  Info:=GlobalLock(BHandle);
  if ( Info<>nil ) then begin
        { calculate ratio of printer/dip resolution }
        if (GetPrnResX(PrinterHandle)<>0) and (GetPrnResY(PrinterHandle)<>0) and
           (GetDibResX(Info)<>0) and (GetDibResY(Info)<>0) then
        begin
          ScaleX:=GetPrnResX(PrinterHandle) / GetDibResX(Info);
          ScaleY:=GetPrnResY(PrinterHandle) / GetDibResY(Info);
        end else begin
          ScaleX:=1;
          ScaleY:=1;
        end;
	    if (ScaleX>1000) or (ScaleY>1000) or (ScaleX<0.001) or (ScaleY<0.001) then
	    begin
          ScaleX:=1;
          ScaleY:=1;
	    end;
        { consider user scale}
        ScaleX:=UserScaleX*ScaleX;
        ScaleY:=UserScaleY*ScaleY;
        { get paper offset }
        if Escape(PrinterHandle, GETPRINTINGOFFSET, 0, NIL, @Offset)<=0 then
           Offset:=point(0,0);
        { center the destination bitmap }
        PageSize:=point(GetDeviceCaps(PrinterHandle, HORZRES), GetDeviceCaps(PrinterHandle, VERTRES));
        w:=round(Info^.biWidth*ScaleX);
        h:=round(Info^.biHeight*ScaleY);
        case Center of
             tctNone: begin
                      X:=0; Y:=0;
                      end;
             tctTopCenter: begin
                           X:=(PageSize.X-w) div 2;
                           Y:=0;
                           end;
             tctCenter: begin
                           X:=(PageSize.X-w) div 2;
                           Y:=(PageSize.Y-h) div 2;
                        end;
             tctBottomCenter: begin
                           X:=(PageSize.X-w) div 2;
                           Y:=(PageSize.Y-h);
                        end;
             else     begin
                      X:=0; Y:=0;
                      end;
        end;
        { draw it on printer canvas }
        i:=StretchDIBits( PrinterHandle,
                          X-Offset.X, Y-Offset.Y, w, h,
                          0, 0, Info^.biWidth, Info^.biHeight,
                          LPBits(Info), PBitmapinfo(Info)^,
                          DIB_RGB_COLORS, SRCCOPY);
  end;
  GlobalUnlock(BHandle);
end;

procedure TForm1.Button1Click(Sender: TObject);
var i, Max : integer;
    Hbmp   : HBitmap;
begin
 Button1.Enabled:=False;
 try
    Hbmp:=TWAIN_AcquireNative(Handle, TWAIN_ANYTYPE); { get bitmap }
    if TWAIN_GetConditionCode<>0 then
    begin
       TWAIN_AbortAllPendingXfers;
    end;
    if Hbmp<>0 then
    begin
     with Printer do begin
         BeginDoc;
         Max:=SpinEdit2.Value;                        { number of copies }
         if Escape(Canvas.Handle, SETCOPYCOUNT, sizeof(Max), @Max, @i)=1 then
            Max:=1;
       try
        for i:=1 to Max do
          PrintDIB( Canvas.Handle, Hbmp,
                    SpinEdit1.Value/100, SpinEdit1.Value/100, { user zoom }
                    PaperSettings.GetCenterState );
       finally;
         EndDoc;
       end;
     end;
    end;
 finally
  if Hbmp<>0 then
  begin
    TWAIN_FreeNative(Hbmp);
  end;
 end;
 Button1.Enabled:=True;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
    UpDateInfo;
end;

procedure TForm1.MPrintersettingsClick(Sender: TObject);
begin
     PrinterSetupDialog1.Execute;
     UpDateInfo;
end;

procedure TForm1.MCloseClick(Sender: TObject);
begin
     Close;
end;

procedure TForm1.MScannersettingsClick(Sender: TObject);
begin
  TWAIN_SelectImageSource(Handle);
  UpDateInfo;
end;

procedure TForm1.Button2Click(Sender: TObject);
var
    Hbmp: HBitmap;
begin
    Hbmp:=TWAIN_AcquireNative(Handle, TWAIN_ANYTYPE);
    TWAIN_WriteNativeToFilename(Hbmp, 'Test.bmp' );
    TWAIN_FreeNative(Hbmp);
end;

procedure TForm1.UpDateInfo;
begin
     Label3.Caption:='Printer: '+Printer.Printers.Strings[Printer.PrinterIndex];
end;

procedure TForm1.MOptionsClick(Sender: TObject);
begin
    PaperSettings.ShowModal;
end;

procedure TForm1.About1Click(Sender: TObject);
begin
    AboutBox.ShowModal;
end;

end.
